@@ -26,7 +26,6 @@ if not os.getenv("READ_THE_DOCS"):
     )
     from cognite.seismic.protos.types_pb2 import (
         CRS,
-        Dimensions,
         ExternalId,
         FileStep,
         GeoJson,
@@ -35,6 +34,7 @@ if not os.getenv("READ_THE_DOCS"):
         PositionQuery,
         Wkt,
     )
+    from cognite.seismic.protos.v1.seismic_service_datatypes_pb2 import Dimensions
     from google.protobuf.empty_pb2 import Empty
 
 
@@ -265,7 +265,6 @@ class FileAPI(API):
         cdp_y_offset: Optional[int] = None,
         source_group_scalar_override: Optional[float] = None,
         external_id: Optional[str] = None,
-        dimensions: int = 3,
     ):
         """Register a file on a survey
 
@@ -303,13 +302,6 @@ class FileAPI(API):
         )
         external_id = None if external_id == None else ExternalId(external_id=external_id)
 
-        if dimensions == 2:
-            dimensions_enum = Dimensions.Value("TWO_DEE")
-        elif dimensions == 3:
-            dimensions_enum = Dimensions.Value("THREE_DEE")
-        else:
-            raise Exception("Invalid `dimensions`. Must be either 2 or 3")
-
         request = RegisterFileRequest(
             survey=survey,
             path=bucket,
@@ -323,7 +315,6 @@ class FileAPI(API):
             cdp_y_offset=cdp_y_offset_wrapped,
             source_group_scalar_override=source_group_scalar_override,
             external_id=external_id,
-            dimensions=dimensions_enum,
         )
         return self.ingestion.RegisterFile(request)
 
