@@ -10,6 +10,7 @@ from cognite.seismic._api.utility import (
     get_identifier,
     get_search_spec,
 )
+from cognite.seismic.data_classes import InternalError, NotFoundError
 from cognite.seismic.data_classes.api_types import BinaryHeader, Geometry, Seismic, TextHeader, VolumeDef
 from google.protobuf.struct_pb2 import Struct
 from google.protobuf.wrappers_pb2 import Int32Value as i32
@@ -18,12 +19,8 @@ from google.protobuf.wrappers_pb2 import StringValue
 if not os.getenv("READ_THE_DOCS"):
     from cognite.seismic.protos.types_pb2 import CRS as CRSProto
     from cognite.seismic.protos.types_pb2 import GeoJson, Wkt
-    from cognite.seismic.protos.v1.seismic_service_datatypes_pb2 import (
-        Identifier,
-        OptionalMap,
-        CoverageSpec,
-        VolumeDef as VolumeDefPB,
-    )
+    from cognite.seismic.protos.v1.seismic_service_datatypes_pb2 import CoverageSpec, Identifier, OptionalMap
+    from cognite.seismic.protos.v1.seismic_service_datatypes_pb2 import VolumeDef as VolumeDefPB
     from cognite.seismic.protos.v1.seismic_service_messages_pb2 import (
         CreateSeismicRequest,
         DeleteSeismicRequest,
@@ -178,7 +175,7 @@ For example, search(coverage_format="wkt") is identical to using include_coverag
             elif mode == "partition":
                 req = SearchSeismicsRequest(partition=spec, coverage=coverage)
             else:
-                raise Exception("mode should be one of: survey, seismic, partition")
+                raise ValueError("mode should be one of: survey, seismic, partition")
 
         req.include_text_header = include_text_header
         req.include_binary_header = include_binary_header
@@ -237,7 +234,7 @@ For example, search(coverage_format="wkt") is identical to using include_coverag
             :py:class:`~cognite.seismic.data_classes.api_types.Seismic`: The matching seismic
         """
         if id is None and external_id is None:
-            raise Exception("Need to provide either the seismic id or external id")
+            raise ValueError("Need to provide either the seismic id or external id")
 
         result = [
             x
@@ -257,13 +254,13 @@ For example, search(coverage_format="wkt") is identical to using include_coverag
         ]
 
         if len(result) > 1:
-            raise Exception("Multiple seismics found. Please contact support")
+            raise InternalError(message="Multiple seismics found. Please contact support")
         elif len(result) == 0:
             if id is not None:
                 msg = f"Seismic with id '{id}'' not found"
             else:
                 msg = f"Seismic with external id '{external_id}' not found"
-            raise Exception(msg)
+            raise NotFoundError(message=msg)
         return result[0]
 
     def edit(
