@@ -14,8 +14,8 @@ if not os.getenv("READ_THE_DOCS"):
 
 
 class SeismicStoreAPI(API):
-    def __init__(self, query, ingestion, metadata):
-        super().__init__(query=query, ingestion=ingestion, metadata=metadata)
+    def __init__(self, query, ingestion):
+        super().__init__(query=query, ingestion=ingestion)
 
     def search(
         self,
@@ -41,7 +41,7 @@ class SeismicStoreAPI(API):
         Only one search method should be specified. The behaviour when multiple are specified is undefined.
 
         Args:
-            mode (str): One of "survey" or "seismic_store".
+            mode (str): One of "survey", "seismic_store", "file".
             id (int|None): Id to search by
             external_id (str|None): External id to search by. NOT IMPLEMENTED
             external_id_substring (str|None): Substring of external id to search by. NOT IMPLEMENTED
@@ -78,14 +78,16 @@ For example, search(coverage_format="wkt") is identical to using include_coverag
                 req = SearchSeismicStoresRequest(seismic_store=spec, coverage=coverage)
             elif mode == "survey":
                 req = SearchSeismicStoresRequest(survey=spec, coverage=coverage)
+            elif mode == "file":
+                req = SearchSeismicStoresRequest(file=spec, coverage=coverage)
             else:
-                raise Exception("mode should be one of: survey, seismic_store")
+                raise Exception("mode should be one of: survey, seismic_store, file")
 
         req.include_file_info = include_file_info
         req.include_headers = include_headers
         req.include_volume_definitions = include_volume_definitions
 
-        results = self.query.SearchSeismicStores(req, metadata=self.metadata)
+        results = self.query.SearchSeismicStores(req)
         for s in results:
             yield SeismicStore.from_proto(s)
 
@@ -157,9 +159,9 @@ For example, search(coverage_format="wkt") is identical to using include_coverag
             :py:class:`~cognite.seismic.data_classes.api_types.SeismicStore`:
         """
         identifier = get_identifier(id, external_id)
-        request = EditSeismicStoreRequest(seismic=identifier)
+        request = EditSeismicStoreRequest(seismic_store=identifier)
         if new_name is not None:
             request.name = new_name
         if metadata is not None:
-            request.metadata = metadata
-        return SeismicStore.from_proto(self.query.EditSeismicStore(request, metadata=self.metadata))
+            request.metadata.MergeFrom(OptionalMap(data=metadata))
+        return SeismicStore.from_proto(self.query.EditSeismicStore(request))
