@@ -3,6 +3,7 @@
 import os
 from typing import *
 
+import deprecation
 from cognite.seismic._api.api import API
 from cognite.seismic._api.utility import get_exact_match_filter, get_identifier
 from cognite.seismic.data_classes.api_types import (
@@ -251,7 +252,6 @@ class SurveyV1API(API):
         Returns:
             :py:class:`~cognite.seismic.data_classes.api_types.Survey`
         """
-
         # Creating the custom coverage field
         if custom_coverage_wkt:
             coverage = CustomSurveyCoverage(custom_coverage=Geometry(crs=crs, wkt=custom_coverage_wkt))
@@ -271,6 +271,7 @@ class SurveyV1API(API):
 
         return Survey.from_proto_survey(self.query.CreateSurvey(request))
 
+    @deprecation.deprecated(deprecated_in="0.2.94", removed_in="0.3.0", details="Use create() instead.")
     def register(
         self,
         name: str,
@@ -284,6 +285,8 @@ class SurveyV1API(API):
     ):
         """Creates a survey with the provided characteristics.
 
+        Deprecated in version 0.2.94. Will be removed in version 0.3.0.
+
         Args:
             survey_name (str): survey name.
             metadata (dict): metadata of the survey.
