@@ -6,17 +6,18 @@ from typing import *
 import numpy as np
 from cognite.seismic._api.api import API
 from cognite.seismic._api.utility import Direction, LineRange, MaybeString, get_identifier, get_search_spec
-from cognite.seismic.data_classes.api_types import RangeInclusive, Trace, VolumeDef
+from cognite.seismic.data_classes.api_types import Geometry, InterpolationMethod, RangeInclusive, Trace, VolumeDef
 
 if not os.getenv("READ_THE_DOCS"):
     from cognite.seismic.protos.types_pb2 import GeoJson
     from cognite.seismic.protos.types_pb2 import Geometry as GeometryProto
-    from cognite.seismic.protos.types_pb2 import LineDescriptor, Wkt
+    from cognite.seismic.protos.types_pb2 import LineDescriptor
     from cognite.seismic.protos.v1.seismic_service_datatypes_pb2 import LineBasedVolume, OptionalMap
     from cognite.seismic.protos.v1.seismic_service_messages_pb2 import (
-        VolumeRequest,
-        SearchSeismicStoresRequest,
+        GeometryBasedVolume,
         SearchSeismicsRequest,
+        SearchSeismicStoresRequest,
+        VolumeRequest,
     )
     from google.protobuf.wrappers_pb2 import Int32Value as i32
     from google.protobuf.wrappers_pb2 import StringValue
@@ -32,18 +33,18 @@ class ArrayData(NamedTuple):
         crs: The coordinate system used
         coord_x: 2D array containing the x coordinate of each (inline, xline) pair
         coord_y: 2D array containing the y coordinate of each (inline, xline) pair
-        inline_range: The range of inline ids described by the first dimension of the array
-        xline_range: The range of xline ids described by the second dimension of the array
-        z_range: The range of depth indices described by the third dimension of the array
+        inline_range: The range of inline ids described by the first dimension of the array, or None if the result is empty
+        xline_range: The range of xline ids described by the second dimension of the array, or None if the result is empty
+        z_range: The range of depth indices described by the third dimension of the array, or None if the result is empty
     """
 
     volume_data: np.ma.MaskedArray
     crs: str
     coord_x: np.ma.MaskedArray
     coord_y: np.ma.MaskedArray
-    inline_range: RangeInclusive
-    xline_range: RangeInclusive
-    z_range: RangeInclusive
+    inline_range: Optional[RangeInclusive]
+    xline_range: Optional[RangeInclusive]
+    z_range: Optional[RangeInclusive]
 
     def __repr__(self) -> str:
         return (
@@ -85,6 +86,8 @@ class VolumeSeismicAPI(API):
         inline_range: Optional[LineRange] = None,
         xline_range: Optional[LineRange] = None,
         z_range: Optional[LineRange] = None,
+        geometry: Optional[Geometry] = None,
+        interpolation_method: Optional[InterpolationMethod] = None,
         include_trace_header: bool = False,
     ) -> Iterable[Trace]:
         """Retrieve traces from a seismic or seismic store
@@ -105,36 +108,20 @@ class VolumeSeismicAPI(API):
         Returns:
             Iterable[:py:class:`~cognite.seismic.data_classes.api_types.Trace`], the traces for the specified volume
         """
-        inline = into_line_range(inline_range)
-        xline = into_line_range(xline_range)
-        zline = into_line_range(z_range)
-        lbs = LineBasedVolume(iline=inline, xline=xline, z=zline)
-        req = VolumeRequest(volume=lbs, include_trace_header=include_trace_header)
-        if seismic_store_id:
-            req.seismic_store_id = seismic_store_id
-        else:
-            req.seismic.MergeFrom(get_identifier(id, external_id))
-
+        req = _build_volume_request(
+            id,
+            external_id,
+            seismic_store_id,
+            inline_range,
+            xline_range,
+            z_range,
+            geometry,
+            interpolation_method,
+            include_trace_header,
+        )
         for proto in self.query.GetVolume(req):
             yield Trace.from_proto(proto)
 
-    def _get_volumedef(
-        self, *, id: Optional[int] = None, external_id: MaybeString = None, seismic_store_id: Optional[int] = None
-    ) -> VolumeDef:
-        """Helper function for get_volume_size and get_array to fetch the volumedef"""
-        if seismic_store_id:
-            spec = get_search_spec(id=seismic_store_id)
-            req = SearchSeismicStoresRequest(seismic_store=spec)
-            req.include_volume_definitions = True
-            [store] = self.query.SearchSeismicStores(req)
-            return VolumeDef.from_proto(store.inline_volume_def)
-        else:
-            spec = get_search_spec(id, external_id)
-            req = SearchSeismicsRequest(seismic=spec)
-            req.include_volume_definition = True
-            [seismic] = self.query.SearchSeismics(req)
-            return VolumeDef.from_proto(seismic.volume_def)
-
     def get_volume_size(
         self,
         *,
@@ -144,6 +131,8 @@ class VolumeSeismicAPI(API):
         inline_range: Optional[LineRange] = None,
         xline_range: Optional[LineRange] = None,
         z_range: Optional[LineRange] = None,
+        geometry: Optional[Geometry] = None,
+        interpolation_method: Optional[InterpolationMethod] = None,
         include_trace_header: bool = False,
     ) -> GetVolumeSize:
         """Estimate the total size of data streamed by get_volume
@@ -153,41 +142,25 @@ class VolumeSeismicAPI(API):
         Returns:
             A :py:class:`~GetVolumeSize` object describing the size
         """
-        volumedef = self._get_volumedef(id=id, external_id=external_id, seismic_store_id=seismic_store_id)
-        # Ignore step sizes
-        if inline_range is not None:
-            inline_range = inline_range[:2]
-        if xline_range is not None:
-            xline_range = xline_range[:2]
-        trace_count = volumedef.count_total_traces(inline_range=inline_range, xline_range=xline_range)
-        sample_count = volumedef.parsed.get("sample_count")
-        if sample_count is not None:
-            if z_range is not None:
-                (z_min, z_max) = z_range[0:2]
-                if z_max < z_min:
-                    raise (Exception("z_range has min > max"))
-                # Clip max to sample_count
-                if z_max > sample_count:
-                    z_max = sample_count
-                # Max may have become smaller than min after clipping
-                if z_max < z_min:
-                    z_min = z_max
-                sample_count = z_max - z_min + 1
+        req = _build_volume_request(
+            id,
+            external_id,
+            seismic_store_id,
+            inline_range,
+            xline_range,
+            z_range,
+            geometry,
+            interpolation_method,
+            include_trace_header,
+        )
+        volume_bounds = self.query.GetVolumeBounds(req)
 
-            # According to segy spec
-            header_size = 240 if include_trace_header else 0
-            int32_size = 4
-            float_size = 4
-            # I guess we will have to do with this estimate?
-            crs_size = len(b"EPSG:23031") + 1  # Null-terminated?
-            coordinate_size = crs_size + 2 * float_size
-            trace_size = header_size + 2 * int32_size + float_size * sample_count + coordinate_size
-            size_bytes = trace_count * trace_size
-            size_kilobytes = round(size_bytes / 1024)
-        else:
-            size_kilobytes = None
+        sample_count = volume_bounds.sample_count
+        size_kilobytes = (volume_bounds.trace_size_bytes * volume_bounds.num_traces) // 1024
 
-        return GetVolumeSize(trace_count=trace_count, sample_count=sample_count, size_kilobytes=size_kilobytes)
+        return GetVolumeSize(
+            trace_count=volume_bounds.num_traces, sample_count=sample_count, size_kilobytes=size_kilobytes
+        )
 
     # Refuse to allocate arrays larger than this
     # FIXME(audunska): Figure out the right limit here, or maybe just use numpy's memory limit
@@ -202,6 +175,8 @@ class VolumeSeismicAPI(API):
         inline_range: Optional[LineRange] = None,
         xline_range: Optional[LineRange] = None,
         z_range: Optional[LineRange] = None,
+        geometry: Optional[Geometry] = None,
+        interpolation_method: Optional[InterpolationMethod] = None,
         progress: Optional[bool] = False,
     ) -> ArrayData:
         """Store traces from a seismic or seismic store into a numpy array
@@ -223,87 +198,64 @@ class VolumeSeismicAPI(API):
             An :py:class:`~ArrayData` object encapsulating the retrieved array (see below)
         """
 
-        volumedef = self._get_volumedef(id=id, external_id=external_id, seismic_store_id=seismic_store_id)
-        z_size = volumedef.parsed.get("sample_count")
+        req = _build_volume_request(
+            id, external_id, seismic_store_id, inline_range, xline_range, z_range, geometry, interpolation_method
+        )
+        volume_bounds = self.query.GetVolumeBounds(req)
+        z_size = volume_bounds.sample_count
 
-        # Compute optimal ranges
-        if inline_range is None:
-            inline_res = volumedef.common_inline_range()
-        else:
-            inline_res = RangeInclusive.from_linerange(inline_range)
-        if xline_range is None:
-            xline_res = volumedef.common_xline_range()
-        else:
-            xline_res = RangeInclusive.from_linerange(xline_range)
-        inline_size = len(inline_res)
-        xline_size = len(xline_res)
-        if z_range is not None:
-            z_res = RangeInclusive.from_linerange(z_range)
-            z_size = len(z_res)
-        elif z_size is not None:
-            z_res = RangeInclusive(0, z_size - 1)
-        else:
-            z_res = None
+        if not volume_bounds.HasField("bounds"):
+            # Not traces to stream, so return early
+            volume_data = np.ma.masked_all((0, 0, z_size), dtype="float")
+            coord_x = np.ma.masked_all((0, 0), dtype="float")
+            coord_y = np.ma.masked_all((0, 0), dtype="float")
+            return ArrayData(
+                volume_data=volume_data,
+                coord_x=coord_x,
+                coord_y=coord_y,
+                crs=volume_bounds.crs,
+                inline_range=None,
+                xline_range=None,
+                z_range=None,
+            )
 
-        def alloc(z_size):
-            if inline_size * xline_size * z_size > self.ARR_LIM:
-                raise ValueError(
-                    f"Array of size ({inline_size},{xline_size},{z_size}) has more than {self.ARR_LIM} elements. Consider restricting the array using inline_range etc."
-                )
-            return np.ma.masked_all((inline_size, xline_size, z_size), dtype="float")
+        bounds = volume_bounds.bounds
+        if not bounds.HasField("iline") or not bounds.HasField("xline"):
+            raise ValueError("get_array not supported for linear geometries")
 
-        if z_size is not None:
-            volume_data = alloc(z_size)
+        inline_range = RangeInclusive.from_proto(bounds.iline)
+        xline_range = RangeInclusive.from_proto(bounds.xline)
+        z_range = RangeInclusive.from_proto(bounds.z)
+
+        inline_size = len(inline_range)
+        xline_size = len(xline_range)
+
+        crs = volume_bounds.crs
+
+        if inline_size * xline_size * z_size > self.ARR_LIM:
+            raise ValueError(
+                f"Array of size ({inline_size},{xline_size},{z_size}) has more than {self.ARR_LIM} elements. Consider restricting the array using inline_range etc."
+            )
+
+        volume_data = np.ma.masked_all((inline_size, xline_size, z_size), dtype="float")
 
-        crs = None
         coord_x = np.ma.masked_all((inline_size, xline_size), dtype="float")
         coord_y = np.ma.masked_all((inline_size, xline_size), dtype="float")
+
         # Fetch data
-        traces = self.get_volume(
-            id=id,
-            external_id=external_id,
-            seismic_store_id=seismic_store_id,
-            inline_range=inline_range,
-            xline_range=xline_range,
-            z_range=z_range,
-            include_trace_header=False,
-        )
+        traces = self.query.GetVolume(req)
+
         if progress:
             try:
                 from tqdm.auto import tqdm
             except ImportError:
                 raise Exception("progress=True requires the tqdm package. Install with 'pip install tqdm'.")
-            traces = tqdm(
-                traces,
-                total=volumedef.count_total_traces(
-                    (inline_res.start, inline_res.stop), (xline_res.start, xline_res.stop)
-                ),
-            )
-        for trace in traces:
-            if z_size is None:
-                z_size = len(trace.trace)
-                z_res = RangeInclusive(0, z_size - 1)
-                # Now we know the trace length, so allocate
-                volume_data = alloc(z_size)
+            traces = tqdm(traces, total=volume_bounds.num_traces)
 
-            # TODO: This slicing should go away once SBS-3251 is fixed
-            trace_slice = trace.trace[:: z_res.step]
-            if len(trace_slice) != z_size:
-                raise Exception(f"Trace length {len(trace.trace)} incompatible with {z_res}")
-            try:
-                inline_ind = inline_res.index(trace.inline)
-                xline_ind = xline_res.index(trace.xline)
-            # TODO: This exception-catching should not be necessary once SBS-3251 is fixed
-            except ValueError as e:
-                if "incompatible" in e.args[0]:
-                    continue
-                else:
-                    raise
-            volume_data[inline_ind, xline_ind, :] = trace_slice
-            if crs is None:
-                crs = trace.coordinate.crs
-            elif trace.coordinate.crs != crs:
-                raise Exception("Incompatible coordinate systems between traces")
+        for trace in traces:
+            inline_ind = inline_range.index(trace.iline.value)
+            xline_ind = xline_range.index(trace.xline.value)
+            volume_data[inline_ind, xline_ind, :] = trace.trace
             coord_x[inline_ind, xline_ind] = trace.coordinate.x
             coord_y[inline_ind, xline_ind] = trace.coordinate.y
         return ArrayData(
@@ -311,9 +263,9 @@ class VolumeSeismicAPI(API):
             crs=crs,
             coord_x=coord_x,
             coord_y=coord_y,
-            inline_range=inline_res,
-            xline_range=xline_res,
-            z_range=z_res,
+            inline_range=inline_range,
+            xline_range=xline_range,
+            z_range=z_range,
         )
 
 
@@ -328,3 +280,35 @@ def into_line_range(linerange: Optional[LineRange]) -> LineDescriptor:
         start, stop, step = linerange
         return LineDescriptor(min=i32(value=start), max=i32(value=stop), step=i32(value=step))
     raise Exception("A line range should be None, (int, int), or (int, int, int).")
+
+
+def _build_volume_request(
+    id: Optional[int] = None,
+    external_id: MaybeString = None,
+    seismic_store_id: Optional[int] = None,
+    inline_range: Optional[LineRange] = None,
+    xline_range: Optional[LineRange] = None,
+    z_range: Optional[LineRange] = None,
+    geometry: Optional[Geometry] = None,
+    interpolation_method: Optional[InterpolationMethod] = None,
+    include_trace_header: Optional[bool] = None,
+) -> "VolumeRequest":
+    zline = into_line_range(z_range)
+    req = VolumeRequest(include_trace_header=include_trace_header)
+    if geometry is not None:
+        if inline_range is not None or xline_range is not None:
+            raise ValueError("Specifying both inline or xline ranges and geometry is not supported")
+        interpolation_method = 0 if interpolation_method is None else interpolation_method.value
+        req.geometry.MergeFrom(
+            GeometryBasedVolume(geometry=geometry.to_proto(), interpolation_method=interpolation_method, z_range=zline)
+        )
+    else:
+        inline = into_line_range(inline_range)
+        xline = into_line_range(xline_range)
+        req.volume.MergeFrom(LineBasedVolume(iline=inline, xline=xline, z=zline))
+
+    if seismic_store_id:
+        req.seismic_store_id = seismic_store_id
+    else:
+        req.seismic.MergeFrom(get_identifier(id, external_id))
+    return req
