@@ -13,7 +13,7 @@ if not os.getenv("READ_THE_DOCS"):
     from cognite.seismic.protos.types_pb2 import DoubleTraceCoordinates as DoubleTraceCoordinatesProto
     from cognite.seismic.protos.types_pb2 import GeoJson
     from cognite.seismic.protos.types_pb2 import Geometry as GeometryProto
-    from cognite.seismic.protos.types_pb2 import IngestionSource
+    from cognite.seismic.protos.types_pb2 import IngestionSource, LineDescriptor
     from cognite.seismic.protos.types_pb2 import P6Transformation as P6TransformationProto
     from cognite.seismic.protos.types_pb2 import SurveyGridTransformation as SurveyGridTransformationProto
     from cognite.seismic.protos.types_pb2 import TraceCorners, Wkt
@@ -22,6 +22,7 @@ if not os.getenv("READ_THE_DOCS"):
     from google.protobuf.json_format import MessageToDict
 
 else:
+    from cognite.seismic._api.shims import LineDescriptor
 
     class IngestionSource(Enum):
         """Enum of ingestion sources."""
@@ -116,10 +117,21 @@ class RangeInclusive:
     def __init__(self, start: int, stop: int, step: Optional[int] = 1):
         if step == 0:
             raise ValueError("RangeInclusive(): step must be nonzero")
+        if step is None:
+            step = 1 if start >= stop else -1
         self.start = start
         self.stop = start + ((stop - start) // step) * step
         self.step = step
 
+    @staticmethod
+    def from_proto(proto: Optional[LineDescriptor]) -> Optional["RangeInclusive"]:
+        if proto is None:
+            return None
+        start = proto.min.value
+        stop = proto.max.value
+        step = proto.step.value if proto.step is not None and proto.step.value is not None else 1
+        return RangeInclusive(start, stop, step)
+
     @classmethod
     def from_linerange(cls, linerange: Optional[LineRange]) -> Optional["RangeInclusive"]:
         """Construct a RangeInclusive from a (start, stop) or (start, stop, step) tuple"""
@@ -639,6 +651,13 @@ class SegyOverrides:
         )
 
 
+class InterpolationMethod(Enum):
+    """Method for interpolating between traces when filtering by linear geometries."""
+
+    NearestTrace = 0
+    InverseDistanceWeighting = 1
+
+
 class Geometry:
     """Represents a CRS + shape, in either a WKT format or a GeoJSON.
 
